@@ -29,8 +29,6 @@
 
 G_BEGIN_DECLS
 
-extern GstElementDetails gst_md5sink_details;
-GST_DEBUG_CATEGORY_EXTERN(gst_md5sink_debug);
 
 #define GST_TYPE_MD5SINK \
   (gst_md5sink_get_type())
@@ -71,8 +69,6 @@ struct _GstMD5SinkClass {
 
 GType				gst_md5sink_get_type		(void);
 
-gboolean 			gst_md5sink_factory_init 	(GstElementFactory *factory);
-
 G_END_DECLS
 
 #endif /* __GST_MD5SINK_H__ */
