@@ -30,6 +30,16 @@
 #include "gstinfo.h"
 #include "gsterror.h"
 
+static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
+static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
 GST_DEBUG_CATEGORY_STATIC (queue_dataflow);
 
 static GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
@@ -106,6 +116,8 @@ static void gst_queue_chain (GstPad * pad, GstData * data);
 static GstData *gst_queue_get (GstPad * pad);
 
 static gboolean gst_queue_handle_src_event (GstPad * pad, GstEvent * event);
+static gboolean gst_queue_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value);
 
 static GstCaps *gst_queue_getcaps (GstPad * pad);
 static GstPadLinkReturn gst_queue_link (GstPad * pad, const GstCaps * caps);
@@ -170,6 +182,10 @@ gst_queue_base_init (GstQueueClass * klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sinktemplate));
   gst_element_class_set_details (gstelement_class, &gst_queue_details);
 }
 
@@ -265,7 +281,9 @@ gst_queue_init (GstQueue * queue)
   GST_FLAG_SET (queue, GST_ELEMENT_DECOUPLED);
   GST_FLAG_SET (queue, GST_ELEMENT_EVENT_AWARE);
 
-  queue->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
+  queue->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&sinktemplate),
+      "sink");
   gst_pad_set_chain_function (queue->sinkpad,
       GST_DEBUG_FUNCPTR (gst_queue_chain));
   gst_element_add_pad (GST_ELEMENT (queue), queue->sinkpad);
@@ -275,7 +293,9 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_active (queue->sinkpad, TRUE);
 
-  queue->srcpad = gst_pad_new ("src", GST_PAD_SRC);
+  queue->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&srctemplate),
+      "src");
   gst_pad_set_get_function (queue->srcpad, GST_DEBUG_FUNCPTR (gst_queue_get));
   gst_element_add_pad (GST_ELEMENT (queue), queue->srcpad);
   gst_pad_set_link_function (queue->srcpad, GST_DEBUG_FUNCPTR (gst_queue_link));
@@ -283,6 +303,8 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_event_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_handle_src_event));
+  gst_pad_set_query_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_handle_src_query));
   gst_pad_set_active (queue->srcpad, TRUE);
 
   queue->cur_level.buffers = 0; /* no content */
@@ -884,6 +906,37 @@ handled:
   return res;
 }
 
+static gboolean
+gst_queue_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value)
+{
+  GstQueue *queue = GST_QUEUE (gst_pad_get_parent (pad));
+  gboolean res;
+
+  if (!GST_PAD_PEER (queue->sinkpad))
+    return FALSE;
+  res = gst_pad_query (GST_PAD_PEER (queue->sinkpad), type, fmt, value);
+  if (!res)
+    return FALSE;
+
+  if (type == GST_QUERY_POSITION) {
+    /* FIXME: this code assumes that there's no discont in the queue */
+    switch (*fmt) {
+      case GST_FORMAT_BYTES:
+        *value -= queue->cur_level.bytes;
+        break;
+      case GST_FORMAT_TIME:
+        *value -= queue->cur_level.time;
+        break;
+      default:
+        /* FIXME */
+        break;
+    }
+  }
+
+  return TRUE;
+}
+
 static gboolean
 gst_queue_release_locks (GstElement * element)
 {
