@@ -71,6 +71,20 @@ gst_structure_get_type (void)
   return gst_structure_type;
 }
 
+static GstStructure *
+gst_structure_id_empty_new_with_size (GQuark quark, guint prealloc)
+{
+  GstStructure *structure;
+
+  structure = g_new0 (GstStructure, 1);
+  structure->type = gst_structure_get_type ();
+  structure->name = quark;
+  structure->fields =
+      g_array_sized_new (FALSE, TRUE, sizeof (GstStructureField), prealloc);
+
+  return structure;
+}
+
 /**
  * gst_structure_id_empty_new:
  * @quark: name of new structure
@@ -82,15 +96,9 @@ gst_structure_get_type (void)
 GstStructure *
 gst_structure_id_empty_new (GQuark quark)
 {
-  GstStructure *structure;
-
   g_return_val_if_fail (quark != 0, NULL);
 
-  structure = g_new0 (GstStructure, 1);
-  structure->name = quark;
-  structure->fields = g_array_new (FALSE, TRUE, sizeof (GstStructureField));
-
-  return structure;
+  return gst_structure_id_empty_new_with_size (quark, 0);
 }
 
 /**
@@ -104,15 +112,9 @@ gst_structure_id_empty_new (GQuark quark)
 GstStructure *
 gst_structure_empty_new (const gchar * name)
 {
-  GstStructure *structure;
-
   g_return_val_if_fail (name != NULL, NULL);
 
-  structure = g_new0 (GstStructure, 1);
-  structure->name = g_quark_from_string (name);
-  structure->fields = g_array_new (FALSE, TRUE, sizeof (GstStructureField));
-
-  return structure;
+  return gst_structure_id_empty_new_with_size (g_quark_from_string (name), 0);
 }
 
 /**
@@ -188,8 +190,9 @@ gst_structure_copy (const GstStructure * structure)
 
   g_return_val_if_fail (structure != NULL, NULL);
 
-  new_structure = gst_structure_empty_new (g_quark_to_string (structure->name));
-  new_structure->name = structure->name;
+  new_structure =
+      gst_structure_id_empty_new_with_size (structure->name,
+      structure->fields->len);
 
   for (i = 0; i < structure->fields->len; i++) {
     GstStructureField new_field = { 0 };
@@ -428,6 +431,13 @@ gst_structure_set_valist (GstStructure * structure,
 
           g_value_init (&field.value, GST_TYPE_BUFFER);
           g_value_set_boxed (&field.value, buffer);
+        } else if (type == GST_TYPE_FRACTION) {
+          gint n, d;
+          n = va_arg (varargs, int);
+          d = va_arg (varargs, int);
+
+          g_value_init (&field.value, GST_TYPE_FRACTION);
+          gst_value_set_fraction (&field.value, n, d);
         } else {
           g_critical ("unimplemented vararg field type %d\n", (int) type);
           return;
@@ -963,9 +973,9 @@ gst_structure_get_string (const GstStructure * structure,
   field = gst_structure_get_field (structure, fieldname);
 
   if (field == NULL)
-    return FALSE;
+    return NULL;
   if (!G_VALUE_HOLDS_STRING (&field->value))
-    return FALSE;
+    return NULL;
 
   return g_value_get_string (&field->value);
 }
@@ -977,43 +987,75 @@ typedef struct _GstStructureAbbreviation
 }
 GstStructureAbbreviation;
 
-static GstStructureAbbreviation gst_structure_abbrs[] = {
-  {"int", G_TYPE_INT},
-  {"i", G_TYPE_INT},
-  {"float", G_TYPE_FLOAT},
-  {"f", G_TYPE_FLOAT},
-  {"double", G_TYPE_DOUBLE},
-  {"d", G_TYPE_DOUBLE},
-//{ "fourcc",   GST_TYPE_FOURCC },
-  {"boolean", G_TYPE_BOOLEAN},
-  {"bool", G_TYPE_BOOLEAN},
-  {"b", G_TYPE_BOOLEAN},
-  {"string", G_TYPE_STRING},
-  {"str", G_TYPE_STRING},
-  {"s", G_TYPE_STRING}
-};
+static GstStructureAbbreviation *
+gst_structure_get_abbrs (gint * n_abbrs)
+{
+  static GstStructureAbbreviation *abbrs = NULL;
+  static gint num = 0;
+
+  if (abbrs == NULL) {
+    /* dynamically generate the array */
+    GstStructureAbbreviation dyn_abbrs[] = {
+      {"int", G_TYPE_INT}
+      ,
+      {"i", G_TYPE_INT}
+      ,
+      {"float", G_TYPE_FLOAT}
+      ,
+      {"f", G_TYPE_FLOAT}
+      ,
+      {"double", G_TYPE_DOUBLE}
+      ,
+      {"d", G_TYPE_DOUBLE}
+      ,
+      {"buffer", GST_TYPE_BUFFER}
+      ,
+      {"fourcc", GST_TYPE_FOURCC}
+      ,
+      {"4", GST_TYPE_FOURCC}
+      ,
+      {"fraction", GST_TYPE_FRACTION}
+      ,
+      {"boolean", G_TYPE_BOOLEAN}
+      ,
+      {"bool", G_TYPE_BOOLEAN}
+      ,
+      {"b", G_TYPE_BOOLEAN}
+      ,
+      {"string", G_TYPE_STRING}
+      ,
+      {"str", G_TYPE_STRING}
+      ,
+      {"s", G_TYPE_STRING}
+    };
+    num = G_N_ELEMENTS (dyn_abbrs);
+    /* permanently allocate and copy the array now */
+    abbrs = g_new0 (GstStructureAbbreviation, num);
+    memcpy (abbrs, dyn_abbrs, sizeof (GstStructureAbbreviation) * num);
+  }
+  *n_abbrs = num;
+
+  return abbrs;
+}
 
 static GType
 gst_structure_from_abbr (const char *type_name)
 {
   int i;
+  GstStructureAbbreviation *abbrs;
+  gint n_abbrs;
 
   g_return_val_if_fail (type_name != NULL, G_TYPE_INVALID);
 
-  for (i = 0; i < G_N_ELEMENTS (gst_structure_abbrs); i++) {
-    if (strcmp (type_name, gst_structure_abbrs[i].type_name) == 0) {
-      return gst_structure_abbrs[i].type;
+  abbrs = gst_structure_get_abbrs (&n_abbrs);
+
+  for (i = 0; i < n_abbrs; i++) {
+    if (strcmp (type_name, abbrs[i].type_name) == 0) {
+      return abbrs[i].type;
     }
   }
 
-  /* FIXME shouldn't be a special case */
-  if (strcmp (type_name, "fourcc") == 0 || strcmp (type_name, "4") == 0) {
-    return GST_TYPE_FOURCC;
-  }
-  if (strcmp (type_name, "buffer") == 0) {
-    return GST_TYPE_BUFFER;
-  }
-
+  /* this is the fallback */
   return g_type_from_name (type_name);
 }
 
@@ -1021,21 +1063,17 @@ static const char *
 gst_structure_to_abbr (GType type)
 {
   int i;
+  GstStructureAbbreviation *abbrs;
+  gint n_abbrs;
 
   g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
 
-  for (i = 0; i < G_N_ELEMENTS (gst_structure_abbrs); i++) {
-    if (type == gst_structure_abbrs[i].type) {
-      return gst_structure_abbrs[i].type_name;
-    }
-  }
+  abbrs = gst_structure_get_abbrs (&n_abbrs);
 
-  /* FIXME shouldn't be a special case */
-  if (type == GST_TYPE_FOURCC) {
-    return "fourcc";
-  }
-  if (type == GST_TYPE_BUFFER) {
-    return "buffer";
+  for (i = 0; i < n_abbrs; i++) {
+    if (type == abbrs[i].type) {
+      return abbrs[i].type_name;
+    }
   }
 
   return g_type_name (type);
@@ -1044,7 +1082,8 @@ gst_structure_to_abbr (GType type)
 static GType
 gst_structure_value_get_generic_type (GValue * val)
 {
-  if (G_VALUE_TYPE (val) == GST_TYPE_LIST || G_VALUE_TYPE (val) == GST_TYPE_FIXED_LIST) {
+  if (G_VALUE_TYPE (val) == GST_TYPE_LIST
+      || G_VALUE_TYPE (val) == GST_TYPE_FIXED_LIST) {
     GArray *array = g_value_peek_pointer (val);
 
     if (array->len > 0) {
@@ -1281,7 +1320,8 @@ gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
 }
 
 static gboolean
-gst_structure_parse_fixed_list (gchar * s, gchar ** after, GValue * value, GType type)
+gst_structure_parse_fixed_list (gchar * s, gchar ** after, GValue * value,
+    GType type)
 {
   return gst_structure_parse_any_list (s, after, value, type,
       GST_TYPE_FIXED_LIST, '<', '>');
