@@ -36,7 +36,8 @@ GST_ELEMENT_DETAILS ("Threaded container",
     "Generic/Bin",
     "Container that creates/manages a thread",
     "Erik Walthinsen <omega@cse.ogi.edu>, "
-    "Benjamin Otte <in7y118@informatik.uni-hamburg.de");
+    "Benjamin Otte <in7y118@informatik.uni-hamburg.de"
+    "Wim Taymans <wim@fluendo.com");
 
 /* Thread signals and args */
 enum
@@ -74,7 +75,7 @@ static GstElementStateReturn gst_thread_change_state (GstElement * element);
 static void gst_thread_child_state_change (GstBin * bin,
     GstElementState oldstate, GstElementState newstate, GstElement * element);
 
-static void gst_thread_sync (GstThread * thread);
+static void gst_thread_sync (GstThread * thread, gboolean is_self);
 
 #ifndef GST_DISABLE_LOADSAVE
 static xmlNodePtr gst_thread_save_thyself (GstObject * object,
@@ -206,6 +207,7 @@ gst_thread_init (GTypeInstance * instance, gpointer g_class)
 
   thread->lock = g_mutex_new ();
   thread->cond = g_cond_new ();
+  thread->iterate_lock = g_mutex_new ();
 
   thread->thread_id = (GThread *) NULL; /* set in NULL -> READY */
   thread->priority = G_THREAD_PRIORITY_NORMAL;
@@ -218,35 +220,18 @@ gst_thread_dispose (GObject * object)
 
   GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "GstThread: dispose");
 
+  /* if we get here, the thread is really stopped as it has released
+   * the last refcount to the thread object, so we can safely free the
+   * mutex and cond vars */
   G_OBJECT_CLASS (parent_class)->dispose (object);
 
   g_assert (GST_STATE (thread) == GST_STATE_NULL);
 
   GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "GstThread: dispose, freeing locks");
 
-  if (thread->thread_id != NULL) {
-    /* thread is still alive */
-    g_mutex_lock (thread->lock);
-    /* thread can be spinning or waiting */
-    GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
-    GST_FLAG_SET (thread, GST_THREAD_STATE_REAPING);
-
-    if (!GST_FLAG_IS_SET (thread, GST_THREAD_STATE_WAITING)) {
-      GST_DEBUG_OBJECT (thread, "wait");
-      g_cond_wait (thread->cond, thread->lock);
-      GST_DEBUG_OBJECT (thread, "done");
-    }
-    /* now wait for the thread to destroy itself */
-    GST_DEBUG_OBJECT (thread, "signal");
-    g_cond_signal (thread->cond);
-    GST_DEBUG_OBJECT (thread, "wait");
-    g_cond_wait (thread->cond, thread->lock);
-    GST_DEBUG_OBJECT (thread, "done");
-    g_mutex_unlock (thread->lock);
-  }
-
   g_mutex_free (thread->lock);
   g_cond_free (thread->cond);
+  g_mutex_free (thread->iterate_lock);
 
   gst_object_replace ((GstObject **) & GST_ELEMENT_SCHED (thread), NULL);
 }
@@ -389,7 +374,7 @@ gst_thread_release_children_locks (GstThread * thread)
  * This function should be called with the thread lock held.
  */
 static void
-gst_thread_sync (GstThread * thread)
+gst_thread_sync (GstThread * thread, gboolean is_self)
 {
   if (thread->thread_id == NULL)
     return;
@@ -397,7 +382,7 @@ gst_thread_sync (GstThread * thread)
   /* need to stop spinning in any case */
   GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
 
-  if (thread == gst_thread_get_current ()) {
+  if (is_self) {
     /* we're trying to sync ourself. Not much we can do here but hope
      * that the thread will stop spinning and  end up in the waiting
      * state */
@@ -434,6 +419,7 @@ gst_thread_change_state (GstElement * element)
   GstThread *thread;
   GstElementStateReturn ret;
   gint transition;
+  gboolean is_self;
 
   g_return_val_if_fail (GST_IS_THREAD (element), GST_STATE_FAILURE);
 
@@ -441,21 +427,26 @@ gst_thread_change_state (GstElement * element)
       gst_element_state_get_name (GST_STATE (element)),
       gst_element_state_get_name (GST_STATE_PENDING (element)));
 
-  transition = GST_STATE_TRANSITION (element);
-
   thread = GST_THREAD (element);
 
+  /* boolean to check if we called the state change in the same thread as
+   * the iterate thread */
+  is_self = (thread == gst_thread_get_current ());
+
   GST_LOG_OBJECT (thread, "grabbing lock");
   g_mutex_lock (thread->lock);
 
-  gst_thread_sync (thread);
+  gst_thread_sync (thread, is_self);
 
-  /* FIXME: (or GStreamers ideas about "threading"): the element variables are
-     commonly accessed by multiple threads at the same time (see bug #111146
-     for an example) */
-  if (transition != GST_STATE_TRANSITION (element)) {
-    g_warning ("inconsistent state information, fix threading please");
-  }
+  /* no iteration is allowed during this state change because an iteration
+   * can cause another state change conflicting with this one */
+  /* do not try to grab the lock if this method is called from the
+   * same thread as the iterate thread, the lock might be held and we 
+   * might deadlock */
+  if (!is_self)
+    g_mutex_lock (thread->iterate_lock);
+
+  transition = GST_STATE_TRANSITION (element);
 
   switch (transition) {
     case GST_STATE_NULL_TO_READY:
@@ -516,7 +507,7 @@ gst_thread_change_state (GstElement * element)
       /* thread was already gone */
       if (thread->thread_id != NULL) {
         thread->thread_id = NULL;
-        if (thread == gst_thread_get_current ()) {
+        if (is_self) {
           /* or should we continue? */
           GST_LOG_OBJECT (thread,
               "Thread %s is destroying itself. Function call will not return!",
@@ -557,6 +548,8 @@ gst_thread_change_state (GstElement * element)
   } else {
     ret = GST_STATE_SUCCESS;
   }
+  if (!is_self)
+    g_mutex_unlock (thread->iterate_lock);
 
   return ret;
 
@@ -568,6 +561,9 @@ error_out:
 
   g_mutex_unlock (thread->lock);
 
+  if (!is_self)
+    g_mutex_unlock (thread->iterate_lock);
+
   return GST_STATE_FAILURE;
 }
 
@@ -583,9 +579,15 @@ gst_thread_child_state_change (GstBin * bin, GstElementState oldstate,
     GstElementState newstate, GstElement * element)
 {
   GstThread *thread = GST_THREAD (bin);
+  gboolean is_self;
+  GstThread *current;
+
+  current = gst_thread_get_current ();
+
+  is_self = (current == thread);
 
   GST_LOG_OBJECT (bin, "(from thread %s) child %s changed state from %s to %s",
-      gst_thread_get_current ()? GST_ELEMENT_NAME (gst_thread_get_current ()) :
+      current ? GST_ELEMENT_NAME (current) :
       "(none)", GST_ELEMENT_NAME (element),
       gst_element_state_get_name (oldstate),
       gst_element_state_get_name (newstate));
@@ -594,7 +596,7 @@ gst_thread_child_state_change (GstBin * bin, GstElementState oldstate,
     parent_class->child_state_change (bin, oldstate, newstate, element);
 
   /* see if we have to wake up the thread now. */
-  if (thread == gst_thread_get_current ()) {
+  if (is_self) {
     GST_LOG_OBJECT (element, "we are in the thread context");
     /* we are the thread, set the spinning flag so that we start spinning
      * next time */
@@ -654,6 +656,7 @@ gst_thread_main_loop (void *arg)
   g_cond_signal (thread->cond);
   GST_LOG_OBJECT (element, "unlocking lock");
 
+  gst_object_ref (GST_OBJECT (thread));
   /* as long as we're not dying we can continue */
   while (!(GST_FLAG_IS_SET (thread, GST_THREAD_STATE_REAPING))) {
     /* in the playing state we need to do something special */
@@ -664,7 +667,9 @@ gst_thread_main_loop (void *arg)
         gboolean status;
 
         g_mutex_unlock (thread->lock);
+        g_mutex_lock (thread->iterate_lock);
         status = gst_bin_iterate (GST_BIN (thread));
+        g_mutex_unlock (thread->iterate_lock);
         g_mutex_lock (thread->lock);
 
         if (!status) {
@@ -675,24 +680,35 @@ gst_thread_main_loop (void *arg)
             GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
           }
         }
+        /* if we hold the last refcount, the app unreffed the
+         * thread and we should stop ASAP */
+        if (G_OBJECT (thread)->ref_count == 1) {
+          GST_DEBUG_OBJECT (thread, "reaping as refcount is only 1");
+          GST_FLAG_SET (thread, GST_THREAD_STATE_REAPING);
+          GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
+        }
       }
     }
-    /* sync section */
-    GST_LOG_OBJECT (thread, "entering sync");
-    GST_DEBUG_OBJECT (thread, "signal");
-    g_cond_signal (thread->cond);
-    GST_DEBUG_OBJECT (thread, "wait");
-    GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
-    GST_FLAG_SET (thread, GST_THREAD_STATE_WAITING);
-    g_cond_wait (thread->cond, thread->lock);
-    GST_FLAG_UNSET (thread, GST_THREAD_STATE_WAITING);
-    GST_LOG_OBJECT (thread, "wait done");
+    /* do not try to sync when we are REAPING */
+    if (!(GST_FLAG_IS_SET (thread, GST_THREAD_STATE_REAPING))) {
+      /* sync section */
+      GST_LOG_OBJECT (thread, "entering sync");
+      GST_DEBUG_OBJECT (thread, "signal");
+      g_cond_signal (thread->cond);
+      GST_DEBUG_OBJECT (thread, "wait");
+      GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
+      GST_FLAG_SET (thread, GST_THREAD_STATE_WAITING);
+      g_cond_wait (thread->cond, thread->lock);
+      GST_FLAG_UNSET (thread, GST_THREAD_STATE_WAITING);
+      GST_LOG_OBJECT (thread, "wait done");
+    }
   }
   GST_LOG_OBJECT (thread, "unlocking lock");
+  thread->thread_id = NULL;
   g_mutex_unlock (thread->lock);
 
-  /* we need to destroy the scheduler here because it has mapped it's
-   * stack into the threads stack space */
+  /* we need to destroy the scheduler here because it might have 
+   * mapped it's stack into the threads stack space */
   sched = GST_ELEMENT_SCHED (thread);
   if (sched)
     gst_scheduler_reset (sched);
@@ -702,6 +718,7 @@ gst_thread_main_loop (void *arg)
   /* signal - we are out */
   GST_LOG_OBJECT (thread, "Thread %p exits main loop", g_thread_self ());
   g_cond_signal (thread->cond);
+  gst_object_unref (GST_OBJECT (thread));
   /* don't assume the GstThread object exists anymore now */
 
   return NULL;
