@@ -1266,7 +1266,7 @@ gst_string_wrap (const char *s)
       *e++ = *t++;
     } else if (*t < 0x20 || *t >= 0x7f) {
       *e++ = '\\';
-      *e++ = '0' + ((*t) >> 6);
+      *e++ = '0' + ((*(guchar *) t) >> 6);
       *e++ = '0' + (((*t) >> 3) & 0x7);
       *e++ = '0' + ((*t++) & 0x7);
     } else {
@@ -1280,6 +1280,48 @@ gst_string_wrap (const char *s)
   return d;
 }
 
+static char *
+gst_string_unwrap (const gchar * s)
+{
+  /* FIXME: do better memory management? */
+  gchar *ret = g_strdup (s);
+  gchar *read = ret, *write = ret;
+
+  if (*read++ != '"') {
+    g_free (ret);
+    return NULL;
+  }
+  while (*read) {
+    if (GST_ASCII_IS_STRING (*read)) {
+      *write++ = *read++;
+    } else if (*read == '"') {
+      break;
+    } else if (*read == '\\') {
+      read++;
+      if (*read >= '0' && *read <= '7') {
+        if (read[1] < '0' || read[1] > '7' || read[2] < '0' || read[2] > '7') {
+          g_free (ret);
+          return NULL;
+        }
+        *write++ = ((read[0] - '0') << 6) +
+            ((read[1] - '0') << 3) + (read[2] - '0');
+        read += 3;
+      } else {
+        *write++ = *read++;
+      }
+    } else {
+      g_free (ret);
+      return NULL;
+    }
+  }
+  if (*read != '"' || read[1] != '\0') {
+    g_free (ret);
+    return NULL;
+  }
+  *write++ = '\0';
+  return ret;
+}
+
 static char *
 gst_value_serialize_string (const GValue * value)
 {
@@ -1289,7 +1331,18 @@ gst_value_serialize_string (const GValue * value)
 static gboolean
 gst_value_deserialize_string (GValue * dest, const char *s)
 {
-  g_value_set_string (dest, s);
+  if (*s != '"') {
+    if (!g_utf8_validate (s, -1, NULL))
+      return FALSE;
+    g_value_set_string (dest, s);
+    return TRUE;
+  } else {
+    gchar *str = gst_string_unwrap (s);
+
+    if (!str)
+      return FALSE;
+    g_value_set_string_take_ownership (dest, str);
+  }
 
   return TRUE;
 }
@@ -2309,8 +2362,8 @@ gst_type_is_fixed (GType type)
       type == GST_TYPE_LIST) {
     return FALSE;
   }
-  if (G_TYPE_IS_FUNDAMENTAL (type) &&
-      type < G_TYPE_MAKE_FUNDAMENTAL (G_TYPE_RESERVED_GLIB_LAST)) {
+  if (G_TYPE_FUNDAMENTAL (type) <=
+      G_TYPE_MAKE_FUNDAMENTAL (G_TYPE_RESERVED_GLIB_LAST)) {
     return TRUE;
   }
   if (type == GST_TYPE_BUFFER || type == GST_TYPE_FOURCC
@@ -2321,6 +2374,41 @@ gst_type_is_fixed (GType type)
   return FALSE;
 }
 
+/**
+ * gst_value_is_fixed:
+ * @value: the #GValue to check
+ *
+ * Tests if the given GValue, if available in a GstStructure (or any other
+ * container) contains a "fixed" (which means: one value) or an "unfixed"
+ * (which means: multiple possible values, such as data lists or data
+ * ranges) value.
+ *
+ * Returns: true if the value is "fixed".
+ */
+
+gboolean
+gst_value_is_fixed (const GValue * value)
+{
+  GType type = G_VALUE_TYPE (value);
+
+  if (type == GST_TYPE_FIXED_LIST) {
+    gboolean fixed = TRUE;
+    gint size, n;
+    const GValue *kid;
+
+    /* check recursively */
+    size = gst_value_list_get_size (value);
+    for (n = 0; n < size; n++) {
+      kid = gst_value_list_get_value (value, n);
+      fixed &= gst_value_is_fixed (kid);
+    }
+
+    return fixed;
+  }
+
+  return gst_type_is_fixed (type);
+}
+
 /************
  * fraction *
  ************/
