@@ -21,30 +21,22 @@
 #include <gst/gstversion.h>
 #include <gst/gstplugin.h>
 
-extern gboolean gst_mem_index_plugin_init 	(GstPlugin *plugin);
-extern gboolean gst_file_index_plugin_init 	(GstPlugin *plugin);
+extern gboolean gst_mem_index_plugin_init (GstPlugin * plugin);
+extern gboolean gst_file_index_plugin_init (GstPlugin * plugin);
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
   gboolean res = TRUE;
 
   res &= gst_mem_index_plugin_init (plugin);
   res &= gst_file_index_plugin_init (plugin);
-  
+
   return res;
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "gstindexers",
-  "GStreamer core indexers",
-  plugin_init,
-  VERSION,
-  GST_LICENSE,
-  GST_PACKAGE,
-  GST_ORIGIN
-)
-
-
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "gstindexers",
+    "GStreamer core indexers",
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
