@@ -2,7 +2,7 @@
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
  *
- * gstscheduler.c: Default scheduling code for most cases
+ * gstoptimalscheduler.c: Default scheduling code for most cases
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -280,7 +280,8 @@ static GstData *gst_opt_scheduler_get_wrapper (GstPad * srcpad);
  * instead of relying on cothreads to do the switch for us.
  */
 #ifndef USE_COTHREADS
-static void gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched);
+static void gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched,
+    GstOptSchedulerGroup * only_group);
 #endif
 
 
@@ -904,6 +905,7 @@ remove_from_group (GstOptSchedulerGroup * group, GstElement * element)
 
   g_assert (group != NULL);
   g_assert (element != NULL);
+  /* this assert also catches the decoupled elements */
   g_assert (GST_ELEMENT_SCHED_GROUP (element) == group);
 
   /* first decrement the links that this group has with other groups through
@@ -920,6 +922,8 @@ remove_from_group (GstOptSchedulerGroup * group, GstElement * element)
   /* if the element was an entry point in the group, clear the group's
    * entry point, and mark it as unknown */
   if (group->entry == element) {
+    GST_DEBUG ("clearing element %p \"%s\" as entry from group %p",
+        element, GST_ELEMENT_NAME (element), group);
     group->entry = NULL;
     group->type = GST_OPT_SCHEDULER_GROUP_UNKNOWN;
   }
@@ -942,25 +946,6 @@ remove_from_group (GstOptSchedulerGroup * group, GstElement * element)
   return group;
 }
 
-/* count number of elements in the group. Have to be careful because
- * decoupled elements are added as entry point but are not added to
- * the elements list */
-static gint
-group_num_elements (GstOptSchedulerGroup * group)
-{
-  gint num;
-
-  num = group->num_elements;
-  /* decoupled elements are not added to the group but are
-   * added as an entry */
-  if (group->entry) {
-    if (GST_ELEMENT_IS_DECOUPLED (group->entry)) {
-      num++;
-    }
-  }
-  return num;
-}
-
 /* check if an element is part of the given group. We have to be carefull
  * as decoupled elements are added as entry but are not added to the elements 
  * list */
@@ -1017,12 +1002,16 @@ setup_group_scheduler (GstOptScheduler * osched, GstOptSchedulerGroup * group)
     wrapper = loop_group_schedule_function;
 
 #ifdef USE_COTHREADS
-  if (!(group->flags & GST_OPT_SCHEDULER_GROUP_SCHEDULABLE)) {
-    do_cothread_create (group->cothread, osched->context,
-        (cothread_func) wrapper, 0, (char **) group);
-  } else {
-    do_cothread_setfunc (group->cothread, osched->context,
-        (cothread_func) wrapper, 0, (char **) group);
+  /* we can only initialize the cothread stuff when we have
+   * a cothread context */
+  if (osched->context) {
+    if (!(group->flags & GST_OPT_SCHEDULER_GROUP_SCHEDULABLE)) {
+      do_cothread_create (group->cothread, osched->context,
+          (cothread_func) wrapper, 0, (char **) group);
+    } else {
+      do_cothread_setfunc (group->cothread, osched->context,
+          (cothread_func) wrapper, 0, (char **) group);
+    }
   }
 #else
   group->schedulefunc = wrapper;
@@ -1181,7 +1170,8 @@ schedule_group (GstOptSchedulerGroup * group)
 
 #ifndef USE_COTHREADS
 static void
-gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched)
+gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched,
+    GstOptSchedulerGroup * only_group)
 {
   GST_LOG_OBJECT (osched, "running queue: %d groups, recursed %d times",
       g_list_length (osched->runqueue),
@@ -1197,11 +1187,14 @@ gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched)
 
   osched->recursion++;
 
-  while (osched->runqueue) {
+  do {
     GstOptSchedulerGroup *group;
     gboolean res;
 
-    group = (GstOptSchedulerGroup *) osched->runqueue->data;
+    if (only_group)
+      group = only_group;
+    else
+      group = (GstOptSchedulerGroup *) osched->runqueue->data;
 
     /* runqueue holds refcount to group */
     osched->runqueue = g_list_remove (osched->runqueue, group);
@@ -1222,7 +1215,7 @@ gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched)
       GST_LOG_OBJECT (osched, "done scheduling group %p", group);
     }
     unref_group (group);
-  }
+  } while (osched->runqueue && !only_group);
 
   GST_LOG_OBJECT (osched, "run queue length after scheduling %d",
       g_list_length (osched->runqueue));
@@ -1262,7 +1255,7 @@ schedule_chain (GstOptSchedulerChain * chain)
         ref_group (group);
         osched->runqueue = g_list_append (osched->runqueue, group);
       }
-      gst_opt_scheduler_schedule_run_queue (osched);
+      gst_opt_scheduler_schedule_run_queue (osched, NULL);
 #endif
 
       GST_LOG ("done scheduling group %p in chain %p", group, chain);
@@ -1455,7 +1448,7 @@ gst_opt_scheduler_get_wrapper (GstPad * srcpad)
       }
 
       GST_LOG ("recursing into scheduler group %p", group);
-      gst_opt_scheduler_schedule_run_queue (osched);
+      gst_opt_scheduler_schedule_run_queue (osched, group);
       GST_LOG ("return from recurse group %p", group);
 
       /* if the other group was disabled we might have to break out of the loop */
@@ -2153,6 +2146,15 @@ group_elements_set_visited (GstOptSchedulerGroup * group, gboolean visited)
       GST_ELEMENT_UNSET_VISITED (element);
     }
   }
+  /* don't forget to set any decoupled entry points that are not accounted for in the
+   * element list (since they belong to two groups). */
+  if (group->entry) {
+    if (visited) {
+      GST_ELEMENT_SET_VISITED (group->entry);
+    } else {
+      GST_ELEMENT_UNSET_VISITED (group->entry);
+    }
+  }
 }
 
 static GList *
@@ -2367,6 +2369,42 @@ rechain_group (GstOptSchedulerGroup * group)
   }
 }
 
+/* make sure that the group does not contain only single element.
+ * Only loop-based groups can contain a single element. */
+static GstOptSchedulerGroup *
+normalize_group (GstOptSchedulerGroup * group)
+{
+  gint num;
+  gboolean have_decoupled = FALSE;
+
+  if (group == NULL)
+    return NULL;
+
+  num = group->num_elements;
+  /* decoupled elements are not added to the group but are
+   * added as an entry */
+  if (group->entry && GST_ELEMENT_IS_DECOUPLED (group->entry)) {
+    num++;
+    have_decoupled = TRUE;
+  }
+
+  if (num == 1 && group->type != GST_OPT_SCHEDULER_GROUP_LOOP) {
+    GST_LOG ("removing last element from group %p", group);
+    if (have_decoupled) {
+      group->entry = NULL;
+      if (group->chain) {
+        GST_LOG ("removing group %p from its chain", group);
+        chain_group_set_enabled (group->chain, group, FALSE);
+        remove_from_chain (group->chain, group);
+      }
+      group = unref_group (group);
+    } else {
+      group = remove_from_group (group, GST_ELEMENT (group->elements->data));
+    }
+  }
+  return group;
+}
+
 /* migrate the element and all connected elements to a new group without looking at
  * the brokenpad */
 static GstOptSchedulerGroup *
@@ -2428,23 +2466,14 @@ group_migrate_connected (GstOptScheduler * osched, GstElement * element,
     /* remove last element from the group if any. Make sure not to remove
      * the loop based entry point of a group as this always needs one group */
     if (group != NULL) {
-      if (group_num_elements (group) == 1 &&
-          group->type != GST_OPT_SCHEDULER_GROUP_LOOP) {
-        GST_LOG ("removing last element from old group");
-        group = remove_from_group (group, GST_ELEMENT (group->elements->data));
-      }
+      group = normalize_group (group);
     }
   }
 
   if (new_group != NULL) {
-    if (group_num_elements (new_group) == 1 &&
-        new_group->type != GST_OPT_SCHEDULER_GROUP_LOOP) {
-      GST_LOG ("removing last element from new group");
-      new_group =
-          remove_from_group (new_group,
-          GST_ELEMENT (new_group->elements->data));
+    new_group = normalize_group (new_group);
+    if (new_group == NULL)
       return NULL;
-    }
     /* at this point the new group lives in its own chain but might
      * have to be merged with another chain, this happens when the new
      * group has a link with another group in another chain */
@@ -2607,6 +2636,8 @@ gst_opt_scheduler_iterate (GstScheduler * sched)
       /* don't schedule any more chains when in error */
       if (osched->state == GST_OPT_SCHEDULER_STATE_ERROR) {
         GST_ERROR_OBJECT (sched, "in error state");
+        /* unref the chain here as we move out of the while loop */
+        unref_chain (chain);
         break;
       } else if (osched->state == GST_OPT_SCHEDULER_STATE_INTERRUPTED) {
         GST_DEBUG_OBJECT (osched, "got interrupted, continue with next chain");
