@@ -6,6 +6,7 @@
 #include <gst/control/control.h>
 #include <string.h>
 #include <locale.h>
+#include <glib/gprintf.h>
 
 #define PUT_START_TAG(pfx,tag)                                  \
 G_STMT_START{                                                   \
@@ -43,6 +44,20 @@ G_STMT_START{                                                   \
   g_print ("%*.*s"str"\n", pfx, pfx, "" , ##a);	   		\
 }G_STMT_END
 
+#else
+
+static inline void
+PUT_STRING (int pfx, const char *format, ...)
+{
+  va_list varargs;
+
+  g_print ("%*.*s", pfx, pfx, "");
+  va_start (varargs, format);
+  g_vprintf (format, varargs);
+  va_end (varargs);
+  g_print ("\n");
+}
+
 #endif
 
 static void
@@ -496,7 +511,7 @@ print_element_info (GstElementFactory * factory)
       else if (padtemplate->direction == GST_PAD_SINK)
         PUT_ESCAPED (3, "direction", "sink");
       else
-        PUT_ESCAPED (3, "direction", "unkown");
+        PUT_ESCAPED (3, "direction", "unknown");
 
       if (padtemplate->presence == GST_PAD_ALWAYS)
         PUT_ESCAPED (3, "presence", "always");
@@ -507,7 +522,7 @@ print_element_info (GstElementFactory * factory)
         PUT_ESCAPED (3, "request-function",
             GST_DEBUG_FUNCPTR_NAME (gstelement_class->request_new_pad));
       } else
-        PUT_ESCAPED (3, "presence", "unkown");
+        PUT_ESCAPED (3, "presence", "unknown");
 
       if (padtemplate->caps) {
         print_caps (padtemplate->caps, 3);
@@ -601,7 +616,7 @@ print_element_info (GstElementFactory * factory)
       else if (gst_pad_get_direction (pad) == GST_PAD_SINK)
         PUT_ESCAPED (3, "direction", "sink");
       else
-        PUT_ESCAPED (3, "direction", "unkown");
+        PUT_ESCAPED (3, "direction", "unknown");
 
       if (GST_IS_GHOST_PAD (pad))
         PUT_ESCAPED (3, "ghost", gst_pad_get_name (pad));
